/**********************************************************
　　製作者　　アドウィン
　　解　説　　SWによるステッピングモータの回転 – プログラム例
**********************************************************/
#include <Arduino.h> // Arduino.h の読み込み
#include "switch.h"  // switch.h の読み込み

// ステッピングモータの動作ピン
int smPins[4] = {2, 5, 16, 17};
// ステッピングモータのパルスパターン
int smPattern[4] = {0b0001, 0b0010, 0b0100, 0b1000};
// ステッピングモータのパルス数
int smPulse = 0;

/*
 * setup 関数
 */
void setup() {
  /* ここにプログラムの最初に1回だけ実行したい処理を上から順に記述していきます */

  // シリアル通信を115200bpsで開始
  Serial.begin(115200);

  Serial.println("初期設定の開始");

  // スイッチの初期設定
  initSwitch();

  // ステッピングモータの動作ピンの初期設定
  for (int i = 0; i < 4; i++) {
    // 出力ピンに設定
    pinMode(smPins[i], OUTPUT);
    // Lレベルに設定
    digitalWrite(smPins[i], LOW);
  }

  Serial.println("初期設定の完了");
}

/*
 * loop 関数
 */
void loop() {
  /* ここにプログラム中で繰り返し実行したい処理を上から順に記述していきます */

  // SW1が押されている時、パルス数を1減らす
  if (SW1_ON && SW2_OFF) {
    smPulse--;
  }
  // SW2が押されている時、パルス数を1増やす
  else if (SW1_OFF && SW2_ON) {
    smPulse++;
  }

  // パルスパターンの取得
  int pattern = smPattern[smPulse & 0b11];
  // 各ピンへのパルスパターンの適用
  for (int i = 0; i < 4; i++) {
    digitalWrite(smPins[i], (pattern >> i) & 1);
  }
  // 脱調防止のための時間待ち
  delay(10);
}