/**********************************************************
　　製作者　　アドウィン
　　解　説　　パルスカウンタ – プログラム例（４逓倍）
**********************************************************/
#include <Arduino.h>      // Arduino.h の読み込み
#include <driver/pcnt.h>  // pcnt.h の読み込み
#include "char8x8.h"      // char8x8.h の読み込み
#include "dotmatrixled.h" // dotmatrixled.h の読み込み

// ロータリエンコーダ用のピン
int encAPin = 26;
int encBPin = 27;

/*
 * setup 関数
 */
void setup() {
  /* ここにプログラムの最初に1回だけ実行したい処理を上から順に記述していきます */

  // シリアル通信を115200bpsで開始
  Serial.begin(115200);

  Serial.println("初期設定の開始");

  // ドットマトリクスLED初期設定
  initLed();

  // ロータリエンコーダ用ピンの初期設定
  pinMode(encAPin, INPUT_PULLUP);
  pinMode(encBPin, INPUT_PULLUP);

  // PCNTの設定用の構造体の宣言（1逓倍）
  pcnt_config_t pcnt_config = {
      .pulse_gpio_num = encAPin,       // パルス入力にA相を設定
      .ctrl_gpio_num = encBPin,        // コントロール信号にB相を設定
      .lctrl_mode = PCNT_MODE_KEEP,    // コントロール信号がLレベルの時、通常のカウント
      .hctrl_mode = PCNT_MODE_DISABLE, // コントロール信号がHレベルの時、カウントしない
      .pos_mode = PCNT_COUNT_INC,      // パルス入力の立ち上がりでカウントを増やす
      .neg_mode = PCNT_COUNT_DEC,      // パルス入力の立ち下がりでカウントを減らす
      .counter_h_lim = 32767,          // カウントの最大値を32767に設定
      .counter_l_lim = -32768,         // カウントの最小値を-32768に設定
      .unit = PCNT_UNIT_0,             // PCNTユニットとしてユニット0を使用
      .channel = PCNT_CHANNEL_0,       // PCNTユニットのチャンネル0を使用
  };
  // 2逓倍用に設定変更
  pcnt_config.hctrl_mode = PCNT_MODE_REVERSE; // コントロール信号がHレベルの時、通常と逆のカウント
  // PCNTの設定（ユニット0チャンネル0）
  pcnt_unit_config(&pcnt_config);
  // 4逓倍用にチャンネル1を設定
  pcnt_config.pulse_gpio_num = encBPin;  // パルス入力をB相に変更
  pcnt_config.ctrl_gpio_num = encAPin;   // コントロール信号をA相に変更
  pcnt_config.pos_mode = PCNT_COUNT_DEC; // パルス入力の立ち上がりでカウントを減らす
  pcnt_config.neg_mode = PCNT_COUNT_INC; // パルス入力の立ち下がりでカウントを増やす
  pcnt_config.channel = PCNT_CHANNEL_1;  // PCNTユニットのチャンネル1を使用
  // PCNTの設定（ユニット0チャンネル1）
  pcnt_unit_config(&pcnt_config);
  // PCNTのユニット0のカウンタ初期化
  pcnt_counter_pause(PCNT_UNIT_0);  // ユニット0のカウンタを一時停止
  pcnt_counter_clear(PCNT_UNIT_0);  // ユニット0のカウンタを初期化
  pcnt_counter_resume(PCNT_UNIT_0); // ユニット0のカウンタを再開

  Serial.println("初期設定の完了");
}

/*
 * loop 関数
 */
void loop() {
  /* ここにプログラム中で繰り返し実行したい処理を上から順に記述していきます */

  // カウント数の取得
  int16_t count;
  pcnt_get_counter_value(PCNT_UNIT_0, &count);

  // シリアルモニタに出力
  Serial.printf(" φ A: %d, φ B %d, カウント数 : %d\n",
  digitalRead(encAPin), digitalRead(encBPin), count);

  // ドットマトリクス LED の設定
  setLed(0xFF, ledCathodePattern[count & 0b111]);

  delay(1); // 時間待ち
}