/**********************************************************
　　製作者　　アドウィン
　　解　説　　ハードウェアPWM – プログラム例
**********************************************************/
#include <Arduino.h>      // Arduino.h の読み込み
#include "dotmatrixled.h" // dotmatrixled.h の読み込み
#include "switch.h"       // switch.h の読み込み

// マイク入力ピン
int micPin = 35;
// DACピン
int dacPin = 25;
// A/D 変換によりできたデジタルデータを格納する配列
byte data[60000];

/*
 * setup 関数
 */
void setup() {
  /* ここにプログラムの最初に1回だけ実行したい処理を上から順に記述していきます */

  // シリアル通信を115200bpsで開始
  Serial.begin(115200);

  Serial.println("初期設定の開始");

  // ドットマトリクスLED初期設定
  initLed();
  // スイッチの初期設定
  initSwitch();

  Serial.println("初期設定の完了");
}

/*
 * loop 関数
 */
void loop() {
  /* ここにプログラム中で繰り返し実行したい処理を上から順に記述していきます */

  // SW1 を押すと、録音を開始
  if (SW1_ON) {

    // チャタリング対策
    delay(10);
    while(SW1_ON) delay(1);
    delay(10);

    // 録音開始（LED右上点灯）
    setLed(0b00000001, 0b00000001);
    Serial.println("録音開始");

    // 録音
    int t0 = micros(); // 開始時刻の記録
    for (int i = 0; i < sizeof(data); i++) {
      // 所定の時刻との差を計算
      int tdiff = i * 125 - (micros() - t0);
      // 所定の時刻より早い時、時間待ちを行う
      if (tdiff > 0) {
        delayMicroseconds(tdiff);
      }
      // ADC入力
      data[i] = analogRead(micPin) >> 4;
    }
    t0 = micros() - t0; // 所要時間の算出
    Serial.println(t0); // 所要時間をシリアル出力

    // 録音終了（LED全消灯）
    setLed(0, 0);
    Serial.println("録音終了");
  }

  // SW2 を押すと、再生を開始
  if (SW2_ON) {

    // チャタリング対策
    delay(10);
    while(SW2_ON) delay(1);;
    delay(10);

    // 再生開始（LED右下点灯）
    setLed(0b00000001, 0b10000000);
    Serial.println("再生開始");

    // 再生
    int t0 = micros(); // 開始時刻の記録
    // 配列データを順番に取得
    for (int i = 0; i < sizeof(data); i++) {
      // 所定の時刻との差を計算
      int tdiff = i * 125 - (micros() - t0);
      // 所定の時刻より早い時、時間待ちを行う
      if (tdiff > 0) {
        delayMicroseconds(tdiff);
      }
      dacWrite(dacPin, data[i]); // DAC 出力
    }
    t0 = micros() - t0; // 所要時間の算出
    Serial.println(t0/1000); // 所要時間をシリアル出力

    // 再生終了（LED全消灯）
    setLed(0, 0);
    Serial.println("再生終了");
  }

  delay(1); // 時間待ち
}